#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2011
# Author: Matthieu Cattin <matthieu.cattin@cern.ch>
# Licence: GPL v2 or later.
# Website: http://www.ohwr.org

import sys
import rr
import time
import os
from numpy import *
from pylab import *

from ptsexcept import *

import gn4124
import fmc_adc
from PAGE.Agilent33250A import *
from PAGE.SineWaveform import *


"""
test09: Test analogue front-end frequency response

Note: Requires test00.py to run first to load the firmware!
"""

GN4124_CSR = 0x0

USB_DEVICE = "/dev/ttyUSB0"
RS232_BAUD = 57600
NB_CHANNELS = 4

AWG_SET_SLEEP = 1
SSR_SET_SLEEP = 0.05

PRE_TRIG_SAMPLES = 1000
POST_TRIG_SAMPLES = 1000
NB_SHOTS = 1

DMA_LENGTH = 4096 # DMA length in bytes

FREQ_POINTS = [1E6, 10E6, 15E6, 16E6, 17E6, 18E6, 19E6, 20E6, 21E6, 22E6, 23E6, 24E6, 25E6, 30E6, 40E6, 60E6, 80E6]


def open_all_channels(fmc):
    for i in range(1,NB_CHANNELS+1):
        fmc.set_input_range(i, 'OPEN')
        time.sleep(SSR_SET_SLEEP)

def set_awg_freq(gen, sine, freq):
    sine.frequency = freq
    gen.play(sine)
    print('Sine frequency:%3.3fMHz')%(sine.frequency/1E6)
    time.sleep(AWG_SET_SLEEP)

def acquisition(gnum, pages, fmc, channel):
    # Start acquisition
    fmc.stop_acq()
    #print('Acquisition FSM state : %s') % fmc.get_acq_fsm_state()
    fmc.start_acq()

    # Wait end of acquisition
    while('IDLE' != fmc.get_acq_fsm_state()):
        #print fmc.get_acq_fsm_state()
        time.sleep(.1)

    # Retrieve data trough DMA
    gnum.add_dma_item(0x100, pages[1], DMA_LENGTH, 0, 0)
    gnum.start_dma()
    gnum.wait_irq()
    page1_data = gnum.get_memory_page(1)
    channels = []
    for i in range(len(page1_data)):
        channels.append(page1_data[i] & 0xFFFF)
        channels.append(page1_data[i]>>16)

    return channels[channel-1::4]


def main (default_directory='.'):

    """
    path_fpga_loader = '../../../gnurabbit/user/fpga_loader';
    path_firmware = '../firmwares/spec_fmcadc100m14b4cha.bin';

    firmware_loader = os.path.join(default_directory, path_fpga_loader)
    bitstream = os.path.join(default_directory, path_firmware)
    print firmware_loader + ' ' + bitstream
    os.system( firmware_loader + ' ' + bitstream )

    time.sleep(2);
    """

    # Objects declaration
    spec = rr.Gennum() # bind to the SPEC board
    gnum = gn4124.CGN4124(spec, GN4124_CSR)
    fmc = fmc_adc.CFmcAdc100Ms(spec)
    gen = Agilent33250A(device=USB_DEVICE, bauds=RS232_BAUD)
    sine = SineWaveform()

    # Set sine params
    sine.frequency = 1E6
    sine.amplitude = 0.25
    sine.dc = 0
    print('Sine frequency:%3.3fMHz amplitude:%2.3fVp offset:%2.3fV')%(sine.frequency/1E6, sine.amplitude, sine.dc)

    # Set AWG
    gen.connect()
    gen.play(sine)
    gen.output = True

    # Reset offset DACs
    fmc.dc_offset_reset()

    # Make sure all switches are OFF
    open_all_channels(fmc)

    # Get physical addresses of the pages for DMA transfer
    pages = gnum.get_physical_addr()

    # Set trigger
    # hw trig, rising edge, external, sw disable, no delay
    fmc.set_trig_config(1, 0, 1, 1, 0, 0, 0)

    # Set acquisition
    fmc.set_pre_trig_samples(PRE_TRIG_SAMPLES)
    fmc.set_post_trig_samples(POST_TRIG_SAMPLES)
    fmc.set_shots(NB_SHOTS)

    ch_diff = []
    for j in range(len(FREQ_POINTS)):
        set_awg_freq(gen, sine, FREQ_POINTS[j])
        for i in range(1,NB_CHANNELS+1):
            fmc.set_input_range(i, '1V')
            time.sleep(SSR_SET_SLEEP)
            channel = acquisition(gnum, pages, fmc, i)
            ch_diff.append(max(channel)-min(channel))
            fmc.set_input_range(i, 'OPEN')

    npoints = arange(0,len(ch_diff)/4,1)
    loglog(FREQ_POINTS, ch_diff[0::4], 'b', FREQ_POINTS, ch_diff[1::4], 'g', FREQ_POINTS, ch_diff[2::4], 'r', FREQ_POINTS, ch_diff[3::4], 'c')
    show()
    raw_input('Press ENTER')

    """
    # Start acquisition
    fmc.stop_acq()
    print('Acquisition FSM state : %s') % fmc.get_acq_fsm_state()
    fmc.start_acq()

    # Wait end of acquisition
    while('IDLE' != fmc.get_acq_fsm_state()):
        print fmc.get_acq_fsm_state()
        time.sleep(1)

    # Retrieve data trough DMA
    gnum.add_dma_item(0*DMA_LENGTH, pages[1]+0*DMA_LENGTH, DMA_LENGTH, 0, 0)
    gnum.start_dma()
    gnum.wait_irq()
    page1_data = gnum.get_memory_page(1)
    channels = []
    for i in range(len(page1_data)):
        channels.append(page1_data[i] & 0xFFFF)
        channels.append(page1_data[i]>>16)
    channel = [[],[],[],[]]
    for i in range(NB_CHANNELS):
        channel[i] = (channels[i::4])
        print('ch%d min:%d max:%d diff:%d')%(i+1, min(channel[i]), max(channel[i]), max(channel[i])-min(channel[i]))
    """

    #for i in range(40):
    #    print("channels:%.4X ch1:%.4X ch2:%.4X ch3:%.4X ch4:%.4X") % (channels[i], channel[0][i/4], channel[1][i/4], channel[2][i/4], channel[3][i/4])

    # Make sure all switches are OFF
    open_all_channels(fmc)

    # Switch AWG OFF
    gen.output = False
    gen.close()


if __name__ == '__main__' :
    main()
